"""sancho.util

   Sancho test utility functions

   Helper functions used to run a single test or multiple tests recursively
   For example, run all tests found in and below the current directory:
   
   >>> print run_all_scripts(find_test_scripts('.'))

"""

__revision__ = "$Id: util.py 20440 2003-01-24 22:38:10Z dbinger $"

import sys, os, re
import new

from sancho.unittest import TestOptions, TestResults, \
        find_scenarios, run_scenarios

def find_test_scripts (start_dir):
    """(start_dir : str) -> [str]

       Recursively explore from 'start_dir', looking for all
       "test/test_*.py" files.  Return the list of matches.
    """

    filename_re = re.compile('^test_\w+\.py$')

    def visit (matches, dirname, names, filename_re=filename_re):
        if os.path.basename(dirname) == 'test':
            for name in names:
                if filename_re.match(name):
                    name = os.path.normpath(os.path.join(dirname, name))
                    matches.append(name)

    matches = []
    os.path.walk(start_dir, visit, matches)
    matches.sort()
    return matches


def run_test_script (script, use_debugger=0, verbosity=1, show_coverage=0): 
    """(script : str,
        use_debugger : boolean=0,
        verbosity : int=1,
        show_coverage : boolean=0)
       -> TestResults

       Run all the tests in file 'script', capture and return results.
    """
    # XXX should catch exceptions from execfile here, but that would
    # mean duplicating logic from TestScenario.run(); I'll wait 'till I
    # have a decent "test collection" abstraction and just do it once.

    test_name = os.path.splitext(os.path.basename(script))[0]
    test_directory = os.path.dirname(script)
    if test_directory and test_directory not in sys.path:
        sys.path = [test_directory] + sys.path
    else:
        test_directory = None
    test_mod = new.module(test_name)
    test_mod.__file__ = script + " (fake)"
    execfile(script, test_mod.__dict__)
    scenarios = find_scenarios(test_mod)

    options = TestOptions(verbosity=verbosity,
                          show_coverage=show_coverage,
                          use_debugger=use_debugger)
    options.show_overall_results = 0
    if verbosity > TestOptions.LOW:
        sys.stdout.write("%s " % script)
    results = run_scenarios(scenarios, options, module=test_mod)
    if test_directory and sys.path[0] == test_directory:
        sys.path = sys.path[1:]
    if verbosity > TestOptions.LOW:
        sys.stdout.write("%s\n" % results)
    return results

def run_all_scripts (scripts, use_debugger=1, show_coverage=0, verbosity=0):
    """(scripts : [str], use_debugger : boolean, verbosity : int)
        -> TestResults

        Run all the test scripts in 'scripts', capture and return results.
    """
    results = TestResults()
    for script in scripts:
        results += run_test_script(script,
                                   use_debugger=use_debugger,
                                   verbosity=verbosity,
                                   show_coverage=show_coverage)
    return results





"""sancho.code_coverage

Provides code coverage analysis for testing
"""

# created 2000/07/06, Neil Schemenauer based on trace.py

__revision__ = "$Id: code_coverage.py 24279 2004-05-19 16:35:56Z dbinger $"

# Example Usage:
#
# import code_coverage
# co = code_coverage.Coverage()
# co.run('main()')
# co.write_results()
#
# or
#
# $ python code_coverage.py "import mymodule; mymodule.main()"

# trace.py
# Copyright 2000, Mojam Media, Inc., all rights reserved.
# Author: Skip Montanaro
#
# Copyright 1999, Bioreason, Inc., all rights reserved.
# Author: Andrew Dalke
#
# Copyright 1995-1997, Automatrix, Inc., all rights reserved.
# Author: Skip Montanaro
#
# Copyright 1991-1995, Stichting Mathematisch Centrum, all rights reserved.
#
#
# Permission to use, copy, modify, and distribute this Python software and
# its associated documentation for any purpose without fee is hereby
# granted, provided that the above copyright notice appears in all copies,
# and that both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of neither Automatrix,
# Bioreason or Mojam Media be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior permission.

import sys, os, string, types, tokenize, token

try:
    from quixote.ptl_compile import Template
    def compile_file(content, filename):
        if filename.endswith('.ptl'):
            template = Template(content, filename)
            template.compile()
            return template.code
        else:
            return compile(content, filename, "exec")
except ImportError:
    def compile_file(content, filename):
        return compile(content, filename, "exec")

def _find_lines_from_code(code, strs):
    """Return dict where keys are lines in the line number table."""
    linenos = {}
    line_increments = [ord(c) for c in code.co_lnotab[1::2]]
    lineno = code.co_firstlineno
    for li in line_increments:
        lineno += li
        if lineno not in strs:
            linenos[lineno] = 1
    return linenos


def _find_lines(code, strs):
    """Return lineno dict for all code objects reachable from code."""
    # get all of the lineno information from the code of this scope level
    linenos = _find_lines_from_code(code, strs)

    # and check the constants for references to other code objects
    for c in code.co_consts:
        if isinstance(c, types.CodeType):
            # find another code object, so recurse into it
            linenos.update(_find_lines(c, strs))
    return linenos


def _find_strings(filename):
    """Return a dict of possible docstring positions.

    The dict maps line numbers to strings.  There is an entry for
    line that contains only a string or a part of a triple-quoted
    string.
    """
    d = {}
    # If the first token is a string, then it's the module docstring.
    # Add this special case so that the test in the loop passes.
    prev_ttype = token.INDENT
    f = open(filename)
    for ttype, tstr, start, end, line in tokenize.generate_tokens(f.readline):
        if ttype == token.STRING:
            if prev_ttype == token.INDENT:
                sline, scol = start
                eline, ecol = end
                for i in range(sline, eline + 1):
                    d[i] = 1
        prev_ttype = ttype
    f.close()
    return d

def find_executable_linenos(filename):
    """Return dict where keys are line numbers in the line number table."""
    try:
        prog = open(filename).read()
    except IOError, err:
        print >> sys.stderr, ("Not printing coverage data for %r: %s"
                              % (filename, err))
        return {}
    code = compile_file(prog, filename)
    strs = _find_strings(filename)
    return _find_lines(code, strs)


class Ignore:
    def __init__(self, modules = None, dirs = None):
        self._mods = modules or []
        self._dirs = dirs or []

        self._ignore = { '<string>': 1 }


    def names(self, filename, modulename):
        if self._ignore.has_key(modulename):
            return self._ignore[modulename]

        # haven't seen this one before, so see if the module name is
        # on the ignore list.  Need to take some care since ignoring
        # "cmp" musn't mean ignoring "cmpcache" but ignoring
        # "Spam" must also mean ignoring "Spam.Eggs".
        for mod in self._mods:
            if mod == modulename:  # Identical names, so ignore
                self._ignore[modulename] = 1
                return 1
            # check if the module is a proper submodule of something on
            # the ignore list
            n = len(mod)
            # (will not overflow since if the first n characters are the
            # same and the name has not already occured, then the size
            # of "name" is greater than that of "mod")
            if mod == modulename[:n] and modulename[n] == '.':
                self._ignore[modulename] = 1
                return 1

        # Now check that __file__ isn't in one of the directories
        if filename is None:
            # must be a built-in, so we must ignore
            self._ignore[modulename] = 1
            return 1

        # Ignore a file when it contains one of the ignorable paths
        for d in self._dirs:
            # The '+ os.sep' is to ensure that d is a parent directory,
            # as compared to cases like:
            #  d = "/usr/local"
            #  filename = "/usr/local.py"
            # or
            #  d = "/usr/local.py"
            #  filename = "/usr/local.py"
            if string.find(filename, d + os.sep) == 0:
                self._ignore[modulename] = 1
                return 1

        # Tried the different ways, so we don't ignore this module
        self._ignore[modulename] = 0
        return 0

class Coverage:
    def __init__(self, modules=None):
        """modules : list of modules for which to show coverage
        default is all modules"""
        self.modules = {} # module -> filename
        self.counts = {}  # (modulename, linenumber) -> count
        if modules is None:
            self.show_all = 1
        else:
            self.show_all = 0
            for module in modules:
                self.modules[module] = None

    def trace(self, frame, why, arg):
        if why == 'line':
            modulename = frame.f_globals.get("__name__", "<unnamed module>")
            if self.show_all or self.modules.has_key(modulename):
                # something is fishy about getting the file name
                filename = frame.f_globals.get("__file__", None)
                if filename is None:
                    filename = frame.f_code.co_filename
                self.modules[modulename] = filename

                lineno = frame.f_lineno

                # record the file name and line number of every trace
                key = (modulename, lineno)
                self.counts[key] = self.counts.get(key, 0) + 1

        return self.trace

    def run(self, cmd):
        import __main__
        dict = __main__.__dict__
        sys.settrace(self.trace)
        try:
            exec cmd in dict, dict
        finally:
            sys.settrace(None)

    def runctx(self, cmd):
        import __main__
        dict = __main__.__dict__
        sys.settrace(self.trace)
        try:
            exec cmd in dict, dict
        finally:
            sys.settrace(None)

    def runfunc(self, func, *args, **kw):
        result = None
        sys.settrace(self.trace)
        try:
            result = apply(func, args, kw)
        finally:
            sys.settrace(None)
        return result

    def write_results(self, outfile=sys.stdout, show_lines=1):
        import re
        # turn the counts data ("(module, lineno) = count") into something
        # accessible on a per-module basis
        per_module = {}
        for modulename, lineno in self.counts.keys():
            try:
                lines_hit = per_module[modulename]
            except KeyError:
                lines_hit = per_module[modulename] = {}
            lines_hit[lineno] = self.counts[(modulename, lineno)]

        # there are many places where this is insufficient, like a blank
        # line embedded in a multiline string.
        blank = re.compile(r'^\s*(#.*)?$')

        for modulename, filename in self.modules.items():
            if filename == '<string>':
                continue
            outfile.write('%s: ' % modulename)
            if filename is None:
                # module as not executed as all
                outfile.write("not executed\n")
                continue
            if filename[-4:] == ".pyc" or filename[-4:] == ".pyo":
                orig_filename = filename[:-4] + '.py'
            elif filename[-5:] == ".ptlc":
                orig_filename = filename[:-1]
            else:
                orig_filename = filename

            if show_lines:
                outfile.write('\n')

            # Get the original lines from the .py file
            try:
                lines = open(orig_filename, 'r').readlines()
            except IOError, err:
                sys.stderr.write(
                    "%s: Could not open %s for reading because: %s - skipping\n" % \
                    ("trace", `filename`, err.strerror))
                continue

            # get a list of the line numbers which represent executable
            # content (returned as a dict for better lookup speed)
            executable_linenos = find_executable_linenos(orig_filename)

            lines_hit = per_module[modulename]
            num_lines = 0
            num_lines_hit = 0
            for i in range(len(lines)):
                line = lines[i]

                # do the blank/comment match to try to mark more lines
                # (help the reader find stuff that hasn't been covered)
                if lines_hit.has_key(i+1):
                    num_lines_hit = num_lines_hit + 1
                    num_lines = num_lines + 1
                    if show_lines:
                        # count precedes the lines that we captured
                        outfile.write('%5d: ' % lines_hit[i+1])
                elif blank.match(line):
                    # blank lines and comments are preceded by dots
                    if show_lines:
                        outfile.write('    . ')
                else:
                    # lines preceded by no marks weren't hit
                    # Highlight them if so indicated, unless the line contains
                    # '#pragma: NO COVER' (it is possible to embed this into
                    # the text as a non-comment; no easy fix)
                    if executable_linenos.has_key(i+1) and \
                       string.find(lines[i], '#pragma NO COVER') == -1:
                        num_lines = num_lines + 1
                        if show_lines:
                            outfile.write('>>>>>> ')
                    else:
                        if show_lines:
                            outfile.write(' '*7)
                if show_lines:
                    outfile.write(string.expandtabs(lines[i], 8))
            if num_lines == 0:
                num_lines = 1 # avoid blowup on empty files
            outfile.write("%3.1f%% (%d/%d)\n" %
                    ((100.0*num_lines_hit)/num_lines, num_lines_hit, num_lines))



def main(stmt):
    co = Coverage()
    co.runctx(stmt)
    co.write_results()
    

if __name__ == '__main__':
    import sys
    if len(sys.argv) == 2:
        main(sys.argv[1])
    else:
        print "Usage: code_coverage.py <statement>"
        sys.exit(1)



"""Sancho: Yet another unit testing framework for Python.
"""

# Mark this directory as a package

# created 2002/01/29, AMK

__revision__ = "$Id: __init__.py 20227 2003-01-16 21:26:56Z akuchlin $"





from sancho.unittest import TestScenario, parse_args, run_scenarios

import simple

tested_modules = ['simple']

class MyFunctionTest (TestScenario):

    def setup(self):
        pass

    def shutdown(self):
        pass

    def check_val_param(self):
        # Negative number should raise a ValueError
        self.test_exc("simple.f('', -1)", ValueError)

        # Zero is OK
        self.test_stmt("simple.f('', 0)")

        # Positive numbers are also OK
        self.test_stmt("simple.f('', 1)")

    def check_func(self):
        # Test the null case (val == 0)
        self.test_val("simple.f('', 0)", '')
        self.test_val("simple.f('abc', 0)", '')

        # Test the identity (val == 1)
        self.test_val("simple.f('', 1)", '')
        self.test_val("simple.f('abc', 1)", 'abc')

        # Test a real case (val == 3)
        self.test_val("simple.f('', 3)", '')
        self.test_val("simple.f('abc', 3)", 'abcabcabc')


if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios(scenarios, options)



#!/usr/bin/env python

__revision__ = "$Id: setup.py 24743 2004-07-21 01:42:01Z dbinger $"

from distutils.core import setup

setup(name = "Sancho",
      version = "0.3",
      description = "Unit testing framework",

      author = "CNRI, (originally Greg Ward)",
      author_email = "webmaster@mems-exchange.org",
      url = "http://www.mems-exchange.org/software/sancho/",
      license = "see LICENSE.txt",
      package_dir = {"sancho": "."},
      packages = ["sancho"],
      scripts = ["scripts/run_sancho_tests"],
     )
